{% extends 'help_base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block content %}

  <h2>{% trans "Status Log" %}</h2>

  <p>
    {% blocktrans trimmed %}
      These are the last {{ num_lines }} lines of the status log for this web
      interface. If you want to report a bug, please use the <a
      href="https://salsa.debian.org/freedombox-team/freedombox/issues">bug
      tracker</a> and attach this status log to the bug report.
    {% endblocktrans %}
  </p>

  <p class="alert alert-warning">
    <span class="fa fa-exclamation-triangle" aria-hidden="true"></span>
    <span class="sr-only">{% trans "Caution:" %}</span>
    {% blocktrans trimmed %}
      Please remove any passwords or other personal information from
      the log before submitting the bug report.
    {% endblocktrans %}
  </p>

  <p>
    <pre class="status-log">{{ data }}</pre>
  </p>

{% endblock %}
