{% extends 'base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block title %}
  <title>
    {% blocktrans trimmed %}
      Page not found - {{ box_name }}
    {% endblocktrans %}
  </title>
{% endblock %}

{% block content %}

  <h2>{% trans "404" %}</h2>

  <p>
    {% blocktrans trimmed %}
      Requested page {{ request_path }} was not found.
    {% endblocktrans %}
  </p>

  <p>
    {% blocktrans trimmed %}
      If you believe this missing page should exist, please file a bug at the
      FreedomBox Service (Plinth) project <a
      href="https://salsa.debian.org/freedombox-team/freedombox/issues">issue
      tracker</a>.
    {% endblocktrans %}
  </p>

{% endblock %}
