{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}

  <h3>{{ title }}</h3>

  <div class="row">
    <div class="col-md-6">
      <div class="list-group list-group-two-column">
        {% for access_point in access_points %}
          <div class="list-group-item">
            <span class="primary">
              {% if access_point.ssid %}
                <a href="{% url 'networks:add_wifi' access_point.ssid access_point.interface_name %}">
                   {{ access_point.ssid }}
                </a>
              {% else %}
                --
              {% endif %}
            </span>

            <span class="btn btn-primary btn-sm secondary">
              {{ access_point.strength }}%
            </span>
          </div>
        {% endfor %}
      </div>
    </div>
  </div>

{% endblock %}
