{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}
  <h3>
    {% blocktrans trimmed with username=object.username %}
      Edit User <em>{{ username }}</em>
    {% endblocktrans %}
  </h3>

  <p>
    {% url 'users:change_password' object.username as change_password_url %}

    {% blocktrans trimmed %}
      Use the <a href='{{ change_password_url }}'>change password form
      </a> to change the password.
    {% endblocktrans %}
  </p>

  <form class="form form-update" method="post">
    {% csrf_token %}

    {{ form|bootstrap }}

    <input type="submit" class="btn btn-primary"
           value="{% trans "Save Changes" %}"/>
  </form>
{% endblock %}

