{% extends "base_firstboot.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block content %}
  {% include "internet_connectivity_content.html" %}

  <form class="form" method="post">
    {% csrf_token %}

    {{ form|bootstrap }}

    <input type="submit" class="btn btn-link" name="skip"
           value="{% trans "Skip this step" %}"/>
    <input type="submit" class="btn btn-primary pull-right" name="next"
           value="{% trans "Next" %}"/>
  </form>
{% endblock %}
