{% extends "base_firstboot.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block body_class %}
no-brand
{% endblock %}

{% block content %}

  {% if is_first_setup_running %}
    <div class="alert alert-warning" role="alert">
      {% blocktrans trimmed %}
        Please wait for {{ box_name }} to finish installation.
        You can start using your {{ box_name }} once it is done.
      {% endblocktrans %}
    </div>
  {% endif %}

  <p class="text-center">
    <img src="{% static 'theme/img/freedombox-logo-standard.svg' %}"
         alt="{{ box_name }}" width="640"/>
  </p>

{% endblock %}
