{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load static %}

{% block page_js %}
  <script type="text/javascript" src="{% static 'mediawiki/mediawiki.js' %}"></script>
{% endblock %}
