{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block content %}

  {% include "app-header.html" with setup=True %}

  {% include "toolbar.html" %}

  {% if setup_state.value == 'up-to-date' %}

     <div class="app-just-installed">
       {% trans "Application installed." %}
     </div>

  {% elif not operations %}

    <p>
      {% if setup_state.value == 'needs-setup' %}
        {% blocktrans trimmed %}
          Install this application?
        {% endblocktrans %}
      {% elif setup_state.value == 'needs-update' %}
        {% blocktrans trimmed %}
          This application needs an update. Update now?
        {% endblocktrans %}
      {% endif %}
    </p>

    <form class="form-install" action="" method="post">
      {% csrf_token %}

      {% if has_unavailable_packages %}
        <div class="alert alert-warning" role="alert">
          {% blocktrans trimmed %}
            This application is currently not available in your distribution.
          {% endblocktrans %}
          <button type="submit" class="btn btn-default btn-sm" name="refresh-packages">
            <span class="fa fa-refresh"></span> {% trans "Check again" %}
          </button>
        </div>
      {% elif package_conflicts and package_conflicts_action.value != 'ignore' %}
        <div class="alert alert-warning" role="alert">
          {% blocktrans trimmed %}
            <strong>Conflicting Packages:</strong> Some packages installed on
            the system conflict with the installation of this app. The following
            packages will be removed if you proceed:
          {% endblocktrans %}
          {% for package in package_conflicts %}
            {{ package }}
          {% endfor %}
        </div>
      {% endif %}

      <input type="submit" class="btn btn-md btn-primary" name="install"
             {% if has_unavailable_packages %}
             disabled="disabled"
             {% endif %}
             {% if setup_state.value == 'needs-setup' %}
             value="{% trans "Install" %}"
             {% elif setup_state.value == 'needs-update' %}
             value="{% trans "Update" %}"
             {% endif %} />

    </form>

  {% else %}
    {% include "operations.html" %}
  {% endif %}

{% endblock %}
