{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later

# This file based on example code from Javascript XMPP Client which is
# licensed as follows.
#
# The MIT License (MIT)
#
# Copyright (c) 2014 Klaus Herberth <klaus@jsxc.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
{% endcomment %}

{% load static %}
{% load i18n %}

<!DOCTYPE HTML>
<html>
  <head>
    <title>Jabber Chat (JSXC)</title>
    <meta http-equiv="content-type" content="text/html; charset=utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1">

    <link href="/javascript/bootstrap4/css/bootstrap.min.css" media="all"
          rel="stylesheet" type="text/css" />
    <link href="/javascript/jquery-ui/themes/base/jquery-ui.min.css"
          media="all" rel="stylesheet" type="text/css" />
    <link href="{% static 'jsxc/libjs-jsxc/jsxc.css' %}" media="all"
          rel="stylesheet" type="text/css" />
    <link href="{% static 'jsxc/jsxc-plinth.css' %}" media="all"
          rel="stylesheet" type="text/css" />

    <script src="/javascript/jquery/jquery.min.js"></script>
    <script src="/javascript/jquery-ui/jquery-ui.min.js"></script>
    <script src="/javascript/jquery-slimscroll/jquery.slimscroll.min.js"></script>
    <script src="/javascript/jquery-fullscreen/jquery.fullscreen.js"></script>
    <script src="/javascript/bootstrap4/js/bootstrap.bundle.min.js"></script>
    <script src="/javascript/jsxc/lib/jsxc.dep.js"></script>
    <script src="/javascript/jsxc/jsxc.js"></script>

    <script src="{% static 'jsxc/jsxc-plinth.js' %}"></script>
  </head>

  <body data-domain="{{ domainname }}"
        data-jsxc-root="{% static 'jsxc/libjs-jsxc' %}">
    <div class="container" id="content" role="main">
      <div class="row">
        <div class="col-lg-6 offset-lg-3">
          <h1>Jabber Chat</h1>

          <div class="form">
            <div class="form-group row">
              <label for="xmpp-domain" class="col-sm-4 col-form-label">
                Domain:
              </label>
              <div class="col-sm-8">
                <input type="text" id="xmpp-domain" name="xmpp-domain"
                       class="form-control" />
                <p id="server-flash"></p>
              </div>
            </div>

            <form id="jsxc-login-form" class="form">
              <fieldset>
                <div class="form-group row">
                  <label for="jsxc-username" class="col-sm-4 col-form-label">
                    Username:
                  </label>
                  <div class="col-sm-8">
                    <input type="text" id="jsxc-username"
                           class="form-control" />
                  </div>
                </div>

                <div class="form-group row">
                  <label for="jsxc-password" class="col-sm-4 col-form-label">
                    Password:
                  </label>
                  <div class="col-sm-8">
                    <input type="password" id="jsxc-password"
                           class="form-control" />
                  </div>
                </div>

                <div class="form-group row">
                  <div class="offset-sm-4 col-sm-8">
                    <button type="submit" id="jsxc-submit"
                            class="submit btn btn-primary">Log in</button>
                    <button class="logout btn btn-secondary">Log out</button>
                  </div>
                </div>
              </fieldset>
            </form>
          </div>
        </div>
      </div>
    </div>

    <div class="d-none">
      <a href="{% static 'jslicense.html' %}" data-jslicense="1">
        {% trans "JavaScript license information" %}</a>
    </div>

  </body>

</html>
