{% extends 'base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load static %}


{% block body_class %}index-page{% endblock %}

{% block container %}

  {% if messages or selected_shortcut or not shortcuts %}
    <div class="container content-container">

      {% include 'messages.html' %}

      {% if not shortcuts %}
        <p>
          {% url 'apps' as apps_url %}
          {% blocktrans trimmed %}
            Enable some <a href="{{ apps_url }}">applications</a> to add
            shortcuts to this page.
          {% endblocktrans %}
        </p>
      {% endif %}

      {% if selected_shortcut.description %}
        {% include "app-header.html" with app_info=selected_shortcut %}

        <section class="toolbar">
          <div class="btn-toolbar" role="toolbar">
            {% include "clients-button.html" with clients=selected_shortcut.clients enabled=True %}

            {% if user.is_authenticated and user_is_admin and selected_shortcut.configure_url %}
              <a class="btn btn-primary" href="{{ selected_shortcut.configure_url }}">
                {% trans "Configure" %}</a>
            {% endif %}
          </div>

          <div class="toolbar-collapsed-content">
            {% include "clients.html" with clients=selected_shortcut.clients enabled=True %}
          </div>
        </section>
      {% endif %}

    </div>
  {% endif %}

  {% if shortcuts %}
    <div class="container card-container">
      <div class="row">
        <div class="card-list">
          {% for shortcut in shortcuts %}
            {% if shortcut.is_enabled %}
              {% if user.is_authenticated or not shortcut.login_required %}
                <div class="card">
                  {% if selected_shortcut.component_id == shortcut.component_id %}
                    <a href="{{ shortcut.url }}" class="nav-link active">
                  {% else %}
                    <a href="{{ shortcut.url }}" class="nav-link">
                  {% endif %}
                        <div class="card-title">
                          {{ shortcut.name }}
                        </div>
                        <div class="card-icon">
                          {% if "custom" in shortcut.icon %}
                            <img src="{{ shortcut.icon }}"/>
                          {% else %}
                            {% if shortcut.app_id %}
                              <img src="{% static shortcut.app_id %}/icons/{{ shortcut.icon }}.svg"/>
                            {% else %}
                              <img src="{% static 'theme/icons/' %}{{ shortcut.icon }}.svg"/>
                            {% endif %}
                          {% endif %}
                        </div>
                        <div class="card-description">
                          {{ shortcut.short_description|default:'' }}
                        </div>
                    </a>
                </div>
              {% endif %}
            {% endif %}
          {% endfor %}
        </div>
      </div>
    </div>
  {% endif %}

{% endblock %}


{% block footer_block %}

  <p>
    <a href="{% url 'index' %}" title="{{ box_name }}">
      <img class="footer-logo" src="{% static 'theme/img/freedombox-logo-standard.svg' %}"
           alt="{{ box_name }}" />
    </a>
  </p>

  <p>
    {% blocktrans trimmed %}
      {{ box_name }}, a Debian pure blend, is a 100% free software
      self-hosting web server to deploy social applications on small
      machines. It provides online communication tools respecting your
      privacy and data ownership.
    {% endblocktrans %}
  </p>

  <p>
    {% blocktrans trimmed %}
      This portal is a part of the {{ box_name }} web interface. {{ box_name }}
      is free software, distributed under the GNU Affero General Public License,
      Version 3 or later.
    {% endblocktrans %}
  </p>

  <div class="row">
    <div class="col-lg-2 offset-lg-3">
      <p>
        <a href="https://wiki.debian.org/FreedomBox/Manual">
          {% trans "Manual" context "User guide"%}
        </a>
      </p>
      <p>
        <a href="https://wiki.debian.org/FreedomBox">{% trans "Wiki" %}</a>
      </p>
      <p>
        <a href="https://salsa.debian.org/freedombox-team/">{% trans "Source Code" %}</a>
      </p>
    </div>
    <div class="col-lg-2">
      <p>
        <a href="https://freedombox.org">{% trans "Homepage" %}</a>
      </p>
      <p>
        <a href="https://freedomboxfoundation.org/donate/">{% trans "Donate" %}</a>
      </p>
      <p>
        <a href="https://freedomboxfoundation.org/">
          {% trans "FreedomBox Foundation" %}
        </a>
      </p>
    </div>
    <div class="col-lg-2">
      <p>
        <a href="https://discuss.freedombox.org/">
          {%trans "Forum" %}
        </a>
      </p>
      <p>
        <a href="https://webchat.oftc.net/?randomnick=1&channels=freedombox&prompt=1">
          {%trans "IRC Chatroom" %}
        </a>
      </p>
      <p>
        <a href="https://lists.alioth.debian.org/mailman/listinfo/freedombox-discuss">
          {% trans "Mailing list" %}
        </a>
      </p>
    </div>
  </div>

{% endblock %}
