{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}
  <h2>{{ title }}</h2>

  {% if expandable_root_size %}
    <p>
      {% blocktrans trimmed %}
        Please backup your data before proceeding.  After this
        operation, {{ expandable_root_size }} of additional free space
        will be available in your root partition.
      {% endblocktrans %}
    </p>
    <form class="form" method="post">
      {% csrf_token %}

      <input type="submit" class="btn btn-primary"
             value="{% trans "Expand Root Partition" %}"/>
    </form>
  {% else %}
    <p>There are no partitions available to expand.
  {% endif %}

{% endblock %}
