{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block content %}
  <h2>{{ title }}</h2>

  {% if name %}
    <p>
      {% trans 'Restore data from' %} {{ name }}
    </p>
  {% endif %}

  <p>
    <form class="form form-restore" method="post">
      {% csrf_token %}

      {{ form|bootstrap }}

      <input id="restore_btn" type="submit"
             class="btn btn-primary"
             value="{% trans 'Restore' %}" />
    </form>
  </p>

{% endblock %}
